/// <reference types="cypress" />
// ***********************************************
// This example commands.ts shows you how to
// create various custom commands and overwrite
// existing commands.
//
// For more comprehensive examples of custom
// commands please read more here:
// https://on.cypress.io/custom-commands
// ***********************************************
//
//
// -- This is a parent command --
// Cypress.Commands.add('login', (email, password) => { ... })
//
//
// -- This is a child command --
// Cypress.Commands.add('drag', { prevSubject: 'element'}, (subject, options) => { ... })
//
//
// -- This is a dual command --
// Cypress.Commands.add('dismiss', { prevSubject: 'optional'}, (subject, options) => { ... })
//
//
// -- This will overwrite an existing command --
// Cypress.Commands.overwrite('visit', (originalFn, url, options) => { ... })
//
// declare global {
//   namespace Cypress {
//     interface Chainable {
//       login(email: string, password: string): Chainable<void>
//       drag(subject: string, options?: Partial<TypeOptions>): Chainable<Element>
//       dismiss(subject: string, options?: Partial<TypeOptions>): Chainable<Element>
//       visit(originalFn: CommandOriginalFn, url: string, options: Partial<VisitOptions>): Chainable<Element>
//     }
//   }
// }


const logout = (): void => {
  cy.log('Logging out by clearing cookies');
  // The order of the cookie clear seems to be important
  // or the logout does not work.
  // First simplesaml backend then keycloak backend.
  // Not further tested and specific reason unknown.
  cy.clearCookies({
    domain: Cypress.env('SIMPLESAML_DOMAIN'),
  });
  cy.clearCookies({
    domain: Cypress.env('KEYCLOAK_DOMAIN'),
  });
  cy.log('Waiting a bit after the cookie clear or the logout/next login is flaky');
  cy.wait(2000);
};

Cypress.Commands.add('login', (user?: {username: string; password: string}): void => {
  const _user = user || {
    username: Cypress.env('DEFAULT_USER_NAME'),
    password: Cypress.env('DEFAULT_USER_PASSWORD'),
  };
  cy.log(`Logging in as: username: ${_user.username}, password: ${_user.password}`);
  // The initial visit to '/' seems to be required
  // or the logout and then next login fails; instead
  // of starting with 'logout()' and then visiting '/'
  // Not further tested and specific reason unknown.
  cy.visit('/');
  cy.log('Logging out (possible) previous user');
  logout();
  cy.visit('/');
  cy.origin('https://ucs-sso.school.test/', {args: {__user: _user}}, ({__user}) => {
    cy.get('input#umcLoginUsername').type(__user.username);
    cy.get('input#umcLoginPassword').type(__user.password);
    cy.get('button.umcLoginFormButton').click();
  });
});

Cypress.Commands.add('logout', (): void => {
  logout();
});
